import unittest
from collections import OrderedDict

from cereja import utils, rescale_values
from cereja.utils.decorators import singleton


class UtilsTest(unittest.TestCase):
    def test_camel_to_snake(self):
        pass

    def test_class_methods(self):
        pass

    def test_combine_with_all(self):
        pass

    def test_fill(self):
        pass

    def test_get_attr_if_exists(self):
        pass

    def test_get_implements(self):
        pass

    def test_get_instances_of(self):
        pass

    def test_import_string(self):
        pass

    def test_install_if_not(self):
        pass

    def test_split_sequence(self):
        val = [10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129,
               130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142]
        expected = [[10, 11, 12, 13, 14, 15, 16, 17, 18, 19],
                    [117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132, 133, 134, 135, 136,
                     137, 138, 139, 140, 141, 142]]
        self.assertListEqual(utils.split_sequence(val, lambda x, y: x + 1 != y), expected)

    def test_invert_dict(self):
        data = {
            "s": 0,
            "y": 1,
            "v": 2,
            "l": 3,
            "i": 4,
            "p": 5,
            "b": 6,
            "z": 7,
            "c": 8,
            "a": 9,
            "k": 10,
            "e": 11,
            "d": 12,
            "j": 13,
            "x": 14,
            "u": 15,
            "o": 16,
            "n": 17,
            "t": 18,
            "f": 19,
            "g": 20,
            "h": 21,
            "r": 22,
            "w": 23,
            "m": 24,
            "q": 25,
        }
        expected = {
            0:  "s",
            1:  "y",
            2:  "v",
            3:  "l",
            4:  "i",
            5:  "p",
            6:  "b",
            7:  "z",
            8:  "c",
            9:  "a",
            10: "k",
            11: "e",
            12: "d",
            13: "j",
            14: "x",
            15: "u",
            16: "o",
            17: "n",
            18: "t",
            19: "f",
            20: "g",
            21: "h",
            22: "r",
            23: "w",
            24: "m",
            25: "q",
        }
        self.assertDictEqual(utils.invert_dict(data), expected)

        data = {1: 40, 2: 30, 3: 40, 30: 2}
        expected = {40: [1, 3], 30: 2, 2: 30}
        self.assertEqual(utils.invert_dict(data), expected)

    def test_logger_level(self):
        pass

    def test_module_references(self):
        pass

    def test_obj_repr(self):
        class Repr:
            def __init__(self, a, b, c):
                self.a = a
                self.b = b
                self.c = c

        expected = (
            "Repr (\n    a = 1,\n    b = dict(oi:<class 'int'> ...),\n    c = oi\n    )"
        )
        self.assertEqual(utils.obj_repr(Repr(1, {"oi": 1}, "oi")), expected)

    def test_rescale_values(self):
        value = list(range(10))
        expected = [0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9]
        self.assertEqual(rescale_values(value, 21), expected)
        expected = [0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9]
        self.assertEqual(rescale_values(value, 22), expected)
        expected = [
            0,
            "joab",
            "joab",
            1,
            "joab",
            "joab",
            2,
            "joab",
            3,
            "joab",
            4,
            "joab",
            5,
            "joab",
            6,
            "joab",
            7,
            "joab",
            8,
            "joab",
            9,
            "joab",
        ]

        self.assertEqual(rescale_values(value, 22, fill_with="joab"), expected)

        expected = [
            0,
            1,
            2,
            3,
            4,
            5,
            6,
            7,
            8,
            9,
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
        ]

        self.assertEqual(
                rescale_values(value, 22, fill_with="joab", filling="post"), expected
        )

        expected = [
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
            "joab",
            0,
            1,
            2,
            3,
            4,
            5,
            6,
            7,
            8,
            9,
        ]

        self.assertEqual(
                rescale_values(value, 22, fill_with="joab", filling="pre"), expected
        )

        expected = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
        self.assertEqual(rescale_values(value, 22, filling="pre"), expected)

        expected = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9]
        self.assertEqual(rescale_values(value, 22, filling="post"), expected)

        expected = [0, 1.8, 3.6, 5.4, 7.2, 9]
        self.assertEqual(rescale_values(value, 6, True), expected)
        expected = [
            0,
            0.5625,
            1.125,
            1.6875,
            2.25,
            2.8125,
            3.375,
            3.9375,
            4.5,
            5.0625,
            5.625,
            6.1875,
            6.75,
            7.3125,
            7.875,
            8.4375,
            9,
        ]
        self.assertEqual(rescale_values(value, 17, True), expected)

    def test_can_do(self):
        class _A:
            def _m1(self):
                pass

            def __m2(self):
                pass

            def m(self):
                pass

            @property
            def m1(self):
                return self._m1()

        a_obj = _A()

        self.assertEqual(utils.can_do(a_obj), ["m", "m1"])

    def test_sample(self):
        data = ["The", "Cereja", "is", "for", "everyone"]

        self.assertEqual(utils.sample(data, 2), ["The", "Cereja"])
        self.assertEqual(utils.sample(data, 3), ["The", "Cereja", "is"])
        self.assertEqual(utils.sample("Cereja"), list("Cereja"))  # test sent string
        self.assertEqual(utils.sample(b"Cereja"), list(b"Cereja"))  # test sent bytes
        self.assertEqual(utils.sample([12], 2), [12])  # test sent number

        data = {"The": "Cereja", "is": "for", "everyone": None}
        self.assertEqual(utils.sample(data, 2), {"The": "Cereja", "is": "for"})

    def test_set_log_level(self):
        pass

    def test_string_to_literal(self):
        pass

    def test_time_format(self):
        pass

    def test_truncate(self):
        self.assertEqual(utils.truncate("Cereja is fun.", k_str=3), "Ce….")
        self.assertEqual(utils.truncate(b"Cereja is fun.", k_iter=3), b"Ce....")
        self.assertRaises(AssertionError, utils.truncate, "Cereja is fun.", -1)
        self.assertEqual(utils.truncate(["Cereja is fun."], k_iter=1000), ["Cereja is fun."])
        self.assertEqual(utils.truncate("Cereja is fun.", k_str=10), "Cerej… fun.")

        self.assertEqual(utils.truncate([0, 1, 2, 3], k_iter=3), [0, 1, '<…>', 3])
        self.assertRaises(AssertionError, utils.truncate, [0, 1, 2, 3], -1)
        self.assertEqual(utils.truncate([0, 1, 2, 3], k_iter=1000), [0, 1, 2, 3])
        self.assertEqual(utils.truncate(list(range(50)), k_iter=5), [0, 1, 2, '<…>', 48, 49])

    def test_type_table_of(self):
        pass

    def test_sort_dict(self):
        val = {0: 1, 1: 2, 2: 1, 3: 4, 4: 1, 5: 43, 6: 1, 7: 10, 8: 22, 9: 0}
        self.assertDictEqual(
                utils.sort_dict(val),
                OrderedDict(
                        [
                            (0, 1),
                            (1, 2),
                            (2, 1),
                            (3, 4),
                            (4, 1),
                            (5, 43),
                            (6, 1),
                            (7, 10),
                            (8, 22),
                            (9, 0),
                        ]
                ),
        )
        self.assertDictEqual(
                utils.sort_dict(val, by_values=True),
                OrderedDict(
                        [
                            (9, 0),
                            (0, 1),
                            (2, 1),
                            (4, 1),
                            (6, 1),
                            (1, 2),
                            (3, 4),
                            (7, 10),
                            (8, 22),
                            (5, 43),
                        ]
                ),
        )
        self.assertDictEqual(
                utils.sort_dict(val, by_values=True, reverse=True),
                OrderedDict(
                        [
                            (5, 43),
                            (8, 22),
                            (7, 10),
                            (3, 4),
                            (1, 2),
                            (0, 1),
                            (2, 1),
                            (4, 1),
                            (6, 1),
                            (9, 0),
                        ]
                ),
        )

        self.assertDictEqual(
                utils.sort_dict(val, by_keys=True),
                OrderedDict(
                        [
                            (0, 1),
                            (1, 2),
                            (2, 1),
                            (3, 4),
                            (4, 1),
                            (5, 43),
                            (6, 1),
                            (7, 10),
                            (8, 22),
                            (9, 0),
                        ]
                ),
        )
        self.assertDictEqual(
                utils.sort_dict(val, by_keys=True, reverse=True),
                OrderedDict(
                        [
                            (9, 0),
                            (8, 22),
                            (7, 10),
                            (6, 1),
                            (5, 43),
                            (4, 1),
                            (3, 4),
                            (2, 1),
                            (1, 2),
                            (0, 1),
                        ]
                ),
        )

    def test_dict_append(self):
        my_dict = {}
        utils.dict_append(my_dict, "key_eg", 1, 2, 3, 4, 5, 6)
        self.assertDictEqual(my_dict, {"key_eg": [1, 2, 3, 4, 5, 6]})
        my_dict = utils.dict_append(my_dict, "key_eg", [1, 2])

        self.assertDictEqual(my_dict, {"key_eg": [1, 2, 3, 4, 5, 6, [1, 2]]})

        my_dict = {}
        utils.dict_append(my_dict, "key_eg", 1, 2, 3, 4, 5, 6, 5, unique_values=True)
        self.assertDictEqual(my_dict, {"key_eg": [1, 2, 3, 4, 5, 6]})

    def test_to_tuple(self):
        data = [
            [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]],
            [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]],
            [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]],
        ]
        expected = (
            ((0.0, 0.0, 0.0), (0.0, 0.0, 0.0), (0.0, 0.0, 0.0)),
            ((0.0, 0.0, 0.0), (0.0, 0.0, 0.0), (0.0, 0.0, 0.0)),
            ((0.0, 0.0, 0.0), (0.0, 0.0, 0.0), (0.0, 0.0, 0.0)),
        )

        self.assertEqual(utils.to_tuple(data), expected)

        data = {
            1: [1, 2, [3, 100, [1000, 30, [12, 3]]], {1: 2, 3: 4}, 4],
            2: [1, 2, 3, 4],
            3: {1, 2, 3},
        }
        expected = (
            (1, (1, 2, (3, 100, (1000, 30, (12, 3))), ((1, 2), (3, 4)), 4)),
            (2, (1, 2, 3, 4)),
            (3, (1, 2, 3)),
        )

        self.assertEqual(utils.to_tuple(data), expected)

    def test_chunk(self):
        tests = [
            ([1, 2, 3, 4, 5, 6], 3, [[1, 2, 3], [4, 5, 6]]),
            ([1, 2, 3, 4, 5, 6], 2, [[1, 2], [3, 4], [5, 6]]),
            ([1, 2, 3, 4, 5, 6], 0, [[1, 2, 3, 4, 5, 6]]),
            ([1, 2, 3, 4, 5, 6, 7], 3, [[1, 2, 3], [4, 5, 6], [7, 0, 0]]),
            ({"oi": "amigo", "tudo": "bem"}, 1, [{"oi": "amigo"}, {"tudo": "bem"}]),
            (
                {"eu": "gosto", "do": "cereja"}.items(),
                1,
                [[("eu", "gosto")], [("do", "cereja")]],
            ),
            (("eu", "gosto", "do", "cereja"), 2, [("eu", "gosto"), ("do", "cereja")]),
        ]

        for test_value, items_per_batch, expected_value in tests:
            print(test_value, items_per_batch, expected_value)
            msg = f"""Test failed for values {test_value}"""
            result = utils.chunk(test_value, batch_size=items_per_batch, fill_with=0)
            self.assertEqual(result, expected_value, msg)

        tests_raise = [([1, 2, 3, 4, 5, 6], "sd", TypeError)]
        for test_value, items_per_batch, expected_error in tests_raise:
            self.assertRaises(expected_error, utils.chunk, test_value, items_per_batch)

    def test_get_batch_strides(self):
        tests = [
            (
                range(1, 26),
                5,
                9,
                [[1, 2, 3, 4, 5], [10, 11, 12, 13, 14], [19, 20, 21, 22, 23]],
            ),
            (
                range(1, 26),
                5,
                2,
                [
                    [1, 2, 3, 4, 5],
                    [3, 4, 5, 6, 7],
                    [5, 6, 7, 8, 9],
                    [7, 8, 9, 10, 11],
                    [9, 10, 11, 12, 13],
                    [11, 12, 13, 14, 15],
                    [13, 14, 15, 16, 17],
                    [15, 16, 17, 18, 19],
                    [17, 18, 19, 20, 21],
                    [19, 20, 21, 22, 23],
                    [21, 22, 23, 24, 25],
                    [23, 24, 25, 25, 25],
                ],
            ),
        ]
        for test_value, kernel_size, strides, expected in tests:
            self.assertEqual(
                    list(utils.get_batch_strides(test_value, kernel_size, strides)),
                    expected,
            )

    def test_value_from_memory(self):
        # Create a test object
        test_object = "Hello, world!"
        # Get the memory ID of the object
        memory_id = id(test_object)
        # Retrieve the object from memory
        retrieved_object = utils.value_from_memory(memory_id)
        # Check if the retrieved object is the same as the original
        self.assertEqual(retrieved_object, test_object)

    def test_combinations(self):
        test_iterable = [1, 2, 3]
        size = 2
        expected_output = [(1, 2), (1, 3), (2, 3)]
        actual_output = utils.combinations(test_iterable, size)
        # Check if the output matches the expected combinations
        self.assertListEqual(sorted(actual_output), sorted(expected_output))

    def test_combinations_sizes(self):
        test_iterable = [3, 2, 1, 4]
        min_size = 2
        max_size = 3
        expected_output = [(2, 3), (1, 3), (3, 4), (1, 2), (2, 4), (1, 4), (1, 2, 3), (2, 3, 4), (1, 3, 4), (1, 2, 4)]
        actual_output = utils.combinations_sizes(test_iterable, min_size, max_size, is_sorted=True)
        # Check if the output includes all combinations for sizes in the given range
        self.assertListEqual(actual_output, expected_output)

        expected_output_unsorted = [(3, 2), (3, 1), (3, 4), (2, 1), (2, 4), (1, 4), (3, 2, 1), (3, 2, 4), (3, 1, 4),
                                    (2, 1, 4)]
        actual_output = utils.combinations_sizes(test_iterable, min_size, max_size, is_sorted=False)
        # Check if the output includes all combinations for sizes in the given range
        self.assertListEqual(actual_output, expected_output_unsorted)


class CjTestTest(unittest.TestCase):
    def test_add_check(self):
        pass

    def test_build_test(self):
        pass

    def test_check_all(self):
        pass

    def test_check_attr(self):
        pass

    def test_parse_attr(self):
        pass

    def test_remove_check(self):
        pass


class SourceTest(unittest.TestCase):
    def test_save(self):
        pass


class DecoratorTest(unittest.TestCase):
    def test_depreciation(self):
        pass

    def test_singleton(self):
        @singleton
        class A:
            def __init__(self, a, b, c=None):
                self.a = a
                self.b = b
                self.c = c

        obj_expected = A(1, b=2, c=3)
        obj_new = A(1, 2)
        self.assertEqual(obj_expected, obj_new, msg="Error on decorator singleton.")

    def test_synchronized(self):
        pass

    def test_thread_safe_generator(self):
        pass

    def test_time_exec(self):
        pass


if __name__ == "__main__":
    unittest.main()



import unittest
import math

from cereja.geolinear import Point


class TestPoint(unittest.TestCase):

    def setUp(self):
        """Setup some common test points."""
        self.p1 = Point(1, 2, 3)
        self.p2 = Point(1, 2, 3)
        self.p3 = Point(4, 5)
        self.p4 = Point(0, 0, 0)
        self.p5 = Point(self.p1)

        self.point = Point(10, 20)

    def test_point_equality(self):
        """Test the equality comparison method."""
        self.assertEqual(self.p1, self.p2)
        self.assertNotEqual(self.p1, self.p3)

    def test_point_copy(self):
        """Test copying a Point object."""
        self.assertEqual(self.p1, self.p5)

    def test_point_inequality(self):
        """Test the inequality comparison method."""
        self.assertNotEqual(self.p1, self.p3)
        self.assertNotEqual(self.p1, self.p4)

    def test_point_less_than(self):
        """Test the less than comparison method."""
        self.assertTrue(self.p4 < self.p1)
        self.assertFalse(self.p1 < self.p2)

    def test_point_less_than_or_equal(self):
        """Test the less than or equal to comparison method."""
        self.assertTrue(self.p4 <= self.p1)
        self.assertTrue(self.p1 <= self.p2)

    def test_point_greater_than(self):
        """Test the greater than comparison method."""
        self.assertTrue(self.p3 > self.p1)
        self.assertFalse(self.p1 > self.p3)

    def test_point_greater_than_or_equal(self):
        """Test the greater than or equal to comparison method."""
        self.assertTrue(self.p3 >= self.p1)
        self.assertTrue(self.p1 >= self.p2)

    def test_magnitude(self):
        """Test the magnitude method."""
        self.assertEqual(self.p4.magnitude, 0)
        self.assertEqual(self.p1.magnitude, math.sqrt(14))

    def test_dot_product(self):
        """Test the dot product method."""
        self.assertEqual(self.p1.dot(self.p2), 14)

    def test_cross_product(self):
        """Test the cross product method for 3D points."""
        result = self.p1.cross(Point(2, 3, 4))
        self.assertEqual(result, Point(-1, 2, -1))

    def test_addition(self):
        """Test addition of points."""
        result = self.p1 + self.p3
        self.assertEqual(result, Point(5, 7, 3))

    def test_subtraction(self):
        """Test subtraction of points."""
        result = self.p1 - self.p2
        self.assertEqual(result, Point(0, 0, 0))

    def test_scalar_multiplication(self):
        """Test scalar multiplication."""
        result = self.p1 * 2
        self.assertEqual(result, Point(2, 4, 6))

    def test_scalar_division(self):
        """Test scalar division."""
        result = self.p1 / 2
        self.assertEqual(result, Point(0.5, 1, 1.5))

    def test_center_position(self):
        # Testar a posição central da bounding box
        bbox = self.point.bounding_box(4, 6, position="center")
        self.assertEqual(bbox, (Point(8, 17), Point(12, 17), Point(12, 23), Point(8, 23)))

    def test_left_top_position(self):
        # Testar a posição left-top da bounding box
        bbox = self.point.bounding_box(4, 6, position="left-top")
        self.assertEqual(bbox, (Point(10, 20), Point(14, 20), Point(14, 26), Point(10, 26)))

    def test_invalid_position(self):
        # Testar posição inválida da bounding box
        with self.assertRaises(ValueError):
            self.point.bounding_box(4, 6, position="invalid_position")


if __name__ == '__main__':
    unittest.main()



import base64

__all__ = ['ColabMediaOutput']


class ColabMediaOutput:
    ITEM_WIDTH = 254
    ITEM_HEIGHT = 254

    _layout = {
        'item': f"""<div class='col-sm text-center'>
<blockquote class="blockquote">
{{media_content}}
<footer><span>{{caption}}</span></footer>
</blockquote>
</div>""",
        'base': f"""
<script src='https://code.jquery.com/jquery-3.2.1.slim.min.js' integrity='sha384-KJ3o2DKtIkvYIK3UENzmM7KCkRr/rE9/Qpg6aAZGJwFDMVNA/GpGFF93hXpG5KkN' crossorigin='anonymous'></script>
<script src='https://cdn.jsdelivr.net/npm/popper.js@1.12.9/dist/umd/popper.min.js' integrity='sha384-ApNbgh9B+Y1QKtv3Rn7W3mgPxhU9K/ScQsAP7hUibX39j7fakFPskvXusvfa0b4Q' crossorigin='anonymous'></script>
<script src='https://cdn.jsdelivr.net/npm/bootstrap@4.0.0/dist/js/bootstrap.min.js' integrity='sha384-JZR6Spejh4U02d8jOt6vLEHfe/JQGiRRSQQxSfFWpi1MquVdAyjUar5+76PVCmYl' crossorigin='anonymous'></script>
<link rel='stylesheet' href='https://cdn.jsdelivr.net/npm/bootstrap@4.0.0/dist/css/bootstrap.min.css' integrity='sha384-Gn5384xqQ1aoWXA+058RXPxPg6fy4IWvTNh0E263XmFcJlSAwiGgFAW/dAiS6JXm' crossorigin='anonymous'>
<div class='container'>
<div class='row'>
{{content}}
</div>
</div>
<script>
    body = document.getElementsByTagName('body')[0]
    body.classList.add('bg-dark')
</script>
"""
    }

    _types = {
        'video': f"""
<video width='{ITEM_WIDTH}' height='{ITEM_HEIGHT}' alt='\\' controls>
    <source src='data:video/mp4;base64,{{media_content_data}}' type=video/mp4 />
</video>
"""

    }

    def __init__(self):
        try:
            from IPython.display import HTML  # noqa: F821
            self.html = HTML
        except ModuleNotFoundError:
            raise Exception('Use only colab')
        self._base = self._layout['base']
        self._content_data = []

    def add_item(self, type_, data, caption=''):
        from cereja.file import FileIO
        from cereja.system import Path
        assert type_ in self._types, f'Only accepted types {tuple(self._types)}'
        if type_ == 'video':
            if isinstance(data, str):
                data = Path(data)
                assert data.suffix == '.mp4', 'Only accepted mp4 type.'
                data = FileIO.load(data).data

            data = base64.b64encode(data).decode("ascii")
            media_content = self._types[type_].format(media_content_data=data)
            media_content = self._layout['item'].format(media_content=media_content, caption=caption)
            self._content_data.append(media_content)

    def show(self):

        html = self._base.format(content=''.join(self._content_data))
        return self.html(
            data=html
        )



import math

__all__ = ["Rotation"]

from typing import Union


class Rotation:
    __axis_map = {0: "x", 1: "y", 2: "z"}

    def __init__(self, degrees: Union[int, float]):
        self._degress = degrees
        self._angle_rad = math.radians(degrees)
        self._cos_angle = math.cos(self._angle_rad)
        self._sin_angle = math.sin(self._angle_rad)
        self._rot_x = [[1, 0, 0], [0, self._cos_angle, -self._sin_angle], [0, self._sin_angle, self._cos_angle]]
        self._rot_y = [[self._cos_angle, 0, self._sin_angle], [0, 1, 0], [-self._sin_angle, 0, self._cos_angle]]
        self._rot_z = [[self._cos_angle, -self._sin_angle, 0], [self._sin_angle, self._cos_angle, 0], [0, 0, 1]]
        self._rot_2d = [[self._cos_angle, -self._sin_angle], [self._sin_angle, self._cos_angle]]

    def __repr__(self):
        return f"{self.__class__.__name__}({self._degress}°)"

    @property
    def rot_3d_x(self):
        return self._rot_x

    @property
    def rot_3d_y(self):
        return self._rot_y

    @property
    def rot_3d_z(self):
        return self._rot_z

    @property
    def rot_2d(self):
        return self._rot_2d

    def rotate_point(self, point, axis=None):
        if len(point) == 2:
            rx = self._rot_2d[0][0] * point[0] + self._rot_2d[0][1] * point[1]
            ry = self._rot_2d[1][0] * point[0] + self._rot_2d[1][1] * point[1]
            return [rx, ry]
        assert len(point) == 3, ValueError(f"{point} isn't 3D point.")
        if axis is None:
            axis = 0
        else:
            assert (axis.lower() if isinstance(axis, str) else axis) in ("x", "y", "z", 0, 1, 2), ValueError(
                    "Invalid axis: choose 'x', 'y', 'z' or 0, 1, 2")
        axis = axis if isinstance(axis, str) else self.__axis_map[int(axis)]
        if axis == "x":
            rot = self.rot_3d_x
        elif axis == "y":
            rot = self.rot_3d_y
        else:
            rot = self.rot_3d_z

        rx = (rot[0][0] * point[0]) + (rot[0][1] * point[1]) + (rot[0][2] * point[2])
        ry = (rot[1][0] * point[0]) + (rot[1][1] * point[1]) + (rot[1][2] * point[2])
        rz = (rot[2][0] * point[0]) + (rot[2][1] * point[1]) + (rot[2][2] * point[2])

        return [rx, ry, rz]

    def rotate(self, val, axis=None):
        from cereja import get_shape, reshape, flatten, shape_is_ok

        if not shape_is_ok(val):
            raise ValueError("value isn't valid to rotate.")

        shape = get_shape(val)
        if len(shape) == 1:
            return self.rotate_point(val, axis=axis)

        return reshape(list(map(lambda point: self.rotate_point(point, axis=axis), flatten(val, depth=len(shape) - 2))),
                       shape)



"""
Copyright (c) 2019 The Cereja Project

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import warnings
import random
import csv

from .. import file as cj_file
from .. import array as cj_array
from ..mltools.pln import LanguageData
from ..system import Path
from .. import utils as cj_utils
from .. import display as cj_display

__all__ = ["Corpus", "make_pairs_by_classes"]


class Corpus(object):
    def __init__(
            self,
            source_data,
            target_data,
            source_name=None,
            percent_train=0.8,
            target_name=None,
            stop_words=(),
            punctuation="!?,.",
            to_lower=True,
            is_remove_punctuation=True,
            is_remove_stop_words=True,
            is_remove_accent=False,
            is_destructive=False,
    ):
        self.source = LanguageData(
                source_data,
                name=source_name,
                stop_words=stop_words,
                punctuation=punctuation,
                to_lower=to_lower,
                is_remove_punctuation=is_remove_punctuation,
                is_remove_stop_words=is_remove_stop_words,
                is_remove_accent=is_remove_accent,
                is_destructive=is_destructive,
        )
        self.target = LanguageData(
                target_data,
                name=target_name,
                stop_words=stop_words,
                punctuation=punctuation,
                to_lower=to_lower,
                is_remove_punctuation=is_remove_punctuation,
                is_remove_stop_words=is_remove_stop_words,
                is_remove_accent=is_remove_accent,
                is_destructive=is_destructive,
        )
        self._percent_train = percent_train
        self._n_train = self.n_train
        self._valid_parallel_data(self.source.data, self.target.data)

    def __iter__(self):
        return zip(self.source, self.target)

    def __len__(self):
        return len(self.source)

    def __repr__(self):
        return f"Corpus(examples: {len(self)} - source_vocab_size: {self.source.vocab_size} - target_vocab_size:{self.target.vocab_size})"

    def __getitem__(self, item):
        if isinstance(item, int):
            return self.source.data[item], self.target.data[item]
        return list(zip(self.source.data[item], self.target.data[item]))

    @property
    def source_language(self):
        """
        Only read. The setter function is `Corpus.source.config.set_config(name="new name")`
        """
        return self.source.config.name

    @property
    def target_language(self):
        """
        Only read. The setter function is `Corpus.target.config.set_config(name="new name")`
        """
        return self.target.config.name

    @property
    def config(self):
        """
        Only read. The setter function is `Corpus.<source or target>.config.set_config`
        """
        return {"source": self.source.config.get(), "target": self.source.config.get()}

    def set_config(self, **kwargs):
        """
        you can use `Corpus.config` which returns a dictionary with the current configuration.
        And then just use the this method `Corpus.set_config` by sending the changed dictionary.

        :param kwargs: dictionary like this:
                    {
                     'source': {'name': 'SOURCE_NAME',
                                'punctuation': '',
                                'remove_punctuation': False,
                                'remove_stop_words': False,
                                'stop_words': (),
                                'to_lower': False
                                },
                     'target': {'name':'TARGET_NAME',
                                'punctuation': '',
                                'remove_punctuation': False,
                                'remove_stop_words': False,
                                'stop_words': (),
                                'to_lower': False
                                }
                    }
        """
        source_config = kwargs.get("source")
        target_config = kwargs.get("target")
        if source_config:
            self.source.config.set_config(**source_config)
        if target_config:
            self.target.config.set_config(**target_config)

    @staticmethod
    def is_parallel(data):
        try:
            for x, y in data:
                if isinstance(x, (str, int, float)) and isinstance(
                        y, (str, int, float)
                ):
                    return True
                break
        except:
            pass
        return False

    @classmethod
    def distinct_from_parallel(cls, data):
        return cj_array.get_cols(data)

    @classmethod
    def load_from_parallel_data(
            cls, data, source_name: str = None, target_name: str = None, **kwargs
    ):
        if cls.is_parallel(data):
            source_data, target_data = cls.distinct_from_parallel(data)
            return cls(
                    source_data,
                    target_data,
                    source_name=source_name,
                    target_name=target_name,
                    **kwargs,
            )
        raise ValueError("isn't valid parallel data")

    @property
    def n_train(self):
        return int(self._percent_train * len(self.source.data))

    @property
    def n_test(self):
        return len(self.source.data) - self.n_train

    def _can_go_test(self, x, y):
        x = self.source.preprocess(x)
        y = self.target.preprocess(y)
        if self.source.phrases_freq.get(x) == 1 and len(x.split()) >= 4:
            x, y = x.split(), y.split()
            for i in x:
                if self.source.words_freq.get(i) <= x.count(i):
                    return False
            for i in y:
                if self.target.words_freq.get(i) <= y.count(i):
                    return False
            return True
        return False

    def _valid_parallel_data(self, x, y):
        assert len(x) == len(
                y
        ), f"Size of {self.source_language} ({len(x)}) != {self.target_language} ({len(y)})"

    def _update_filters(self, x, y):
        x = self.source.preprocess(x)
        y = self.target.preprocess(y)
        for i in x.split():
            self.source.words_freq.subtract([i])

        for i in y.split():
            self.target.words_freq.subtract([i])

        self.source.phrases_freq.subtract([x])
        self.target.phrases_freq.subtract([y])

    def _get_vocab_data(
            self,
            source_vocab_size: int = None,
            target_vocab_size: int = None,
            order="most_common",
    ):
        source_vocab_data = {}
        target_vocab_data = {}
        if source_vocab_size is not None:
            source_vocab_data = self.source.sample_words_freq(
                    max_items=source_vocab_size, order=order
            )
        if target_vocab_size is not None:
            target_vocab_data = self.target.sample_words_freq(
                    max_items=target_vocab_size, order=order
            )

        for x, y in zip(self.source.data, self.target.data):
            if source_vocab_size:
                if not all(
                        list(
                                map(
                                        lambda w: w in source_vocab_data,
                                        self.source.preprocess(x).split(),
                                )
                        )
                ):
                    continue
            if target_vocab_size:
                if not all(
                        list(
                                map(
                                        lambda w: w in target_vocab_data,
                                        self.target.preprocess(y).split(),
                                )
                        )
                ):
                    continue
            yield [x, y]

    def save(
            self,
            save_on_dir: str,
            take_split: bool = True,
            test_max_size: int = None,
            source_vocab_size: int = None,
            target_vocab_size: int = None,
            shuffle=True,
            prefix=None,
            ext="align",
            **kwargs,
    ):
        save_on_dir = Path(save_on_dir)
        if take_split:
            x_train, y_train, x_test, y_test = self.split_data(
                    test_max_size=test_max_size,
                    source_vocab_size=source_vocab_size,
                    target_vocab_size=target_vocab_size,
                    take_parallel_data=False,
                    shuffle=shuffle,
            )
            train_prefix, test_prefix = (
                (f"{prefix}_train", f"{prefix}_test")
                if prefix is not None
                else ("train", "test")
            )
            data_to_save = (
                (train_prefix, x_train, y_train),
                (test_prefix, x_test, y_test),
            )
        else:
            data_to_save = ((prefix, self.source.data, self.target.data),)

        for prefix, x, y in data_to_save:
            save_on = save_on_dir.join(
                    f'{prefix}_{self.source_language}.{ext.strip(".")}'
            )
            cj_file.FileIO.create(save_on, data=x).save(**kwargs)
            save_on = save_on_dir.join(
                    f'{prefix}_{self.target_language}.{ext.strip(".")}'
            )
            cj_file.FileIO.create(save_on, data=y).save(**kwargs)

    @classmethod
    def load_corpus_from_csv(
            cls,
            path_: str,
            src_col_name: str,
            trg_col_name: str,
            source_name=None,
            target_name=None,
    ):

        csv_read = csv.DictReader(cj_file.FileIO.load(path_).data)
        src_data = []
        trg_data = []
        for i in csv_read:
            for col_name in (src_col_name, trg_col_name):
                if col_name not in i:
                    raise ValueError(f"Not found col <{col_name}> in {list(i.keys())}")
            src_data.append(i[src_col_name])
            trg_data.append(i[trg_col_name])
        return cls(src_data, trg_data, source_name=source_name, target_name=target_name)

    def split_data(
            self,
            test_max_size: int = None,
            source_vocab_size: int = None,
            target_vocab_size: int = None,
            shuffle=True,
            take_parallel_data=True,
            take_corpus_instances=False,
            legacy_test=None,
    ):
        """
        Guarantees test data without data identical to training and only with vocabulary that exists in training


        :param test_max_size: int = max examples on test data
        :param source_vocab_size: int = restrict most common vocab
        :param target_vocab_size: int = restrict most common vocab
        :param shuffle: bool = randomize
        :param take_parallel_data: bool = zipped data if true else return (x_train, y_train, x_test, y_test)
        :param take_corpus_instances: bool = return new instances for train data and test data
        :param legacy_test: List[Tuple[str,str]] = parallel data
        """
        self.source.reset_freq()
        self.target.reset_freq()
        train = []
        test = []

        if legacy_test is not None:
            test = Corpus(
                    *self.distinct_from_parallel(legacy_test),
                    source_name=self.source_language,
                    target_name=self.target_language,
            )

        test_max_size = (
            test_max_size
            if test_max_size is not None and isinstance(test_max_size, (int, float))
            else len(self.source.data) - self.n_train
        )
        if source_vocab_size is not None or target_vocab_size is not None:
            data = list(
                    self._get_vocab_data(
                            source_vocab_size=source_vocab_size,
                            target_vocab_size=target_vocab_size,
                    )
            )
        else:
            data = list(zip(self.source.data, self.target.data))

        if shuffle:
            random.shuffle(data)

        for x, y in data:
            # remove blank line
            if x == "" or y == "":
                continue
            if legacy_test is not None:
                # remove sentence from train.
                if self.source.preprocess(x) in test.source.phrases_freq:
                    continue
            if (
                    self._can_go_test(x, y) and len(test) < test_max_size
            ) and legacy_test is None:
                test.append([x, y])
                self._update_filters(x, y)
                continue
            train.append([x, y])

        if take_parallel_data is False:
            return (*cj_array.get_cols(train), *cj_array.get_cols(test))
        if take_corpus_instances is True:
            train = self.load_from_parallel_data(
                    train, self.source_language, self.target_language
            )
            test = self.load_from_parallel_data(
                    test, self.source_language, self.target_language
            )
            return train, test
        return train, test

    def split_data_and_save(self, **kwargs):
        alternative = f"You can use <Corpus.save>"
        warnings.warn(
                f"This function has been deprecated and will be removed in future versions. "
                f"{alternative}",
                DeprecationWarning,
                2,
        )
        self.save(**kwargs)


def make_pairs_by_classes(clusters, limit_items_per_batch=None, limit_n_clusters=None):
    result = []
    if limit_items_per_batch or limit_n_clusters:
        clusters = {
            i: cj_utils.sample(values, k=limit_items_per_batch, is_random=True)
            for i, values in cj_utils.sample(clusters, k=limit_n_clusters).items()
            if len(values)
        }
    all_classes = list(clusters)
    for primary_id, items_on_cluster in cj_display.Progress.prog(clusters.items()):
        if len(items_on_cluster) == 1:
            continue
        all_items_on_cluster = {primary_id, *items_on_cluster}

        result += [
            (a, b, 1)
            for a, b in cj_utils.combine_with_all(
                    *cj_utils.chunk(all_items_on_cluster, max_batches=2)
            )
        ]
    result += [
        (a, b, 0)
        for a, b in cj_utils.combine_with_all(
                *cj_utils.chunk(all_classes, max_batches=2)
        )
    ]
    random.shuffle(result)
    return result



"""
Copyright (c) 2019 The Cereja Project

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
from typing import Union, Sequence, Optional, Tuple
import math

__all__ = ["Point"]


class Point:
    """
    Represents a point in a 2D or 3D space.

    Attributes:
        x (Union[int, float]): X-coordinate of the point.
        y (Union[int, float]): Y-coordinate of the point.
        z (Union[int, float]): Z-coordinate of the point. Defaults to 0 for 2D points.
        _is_3d (bool): A private attribute indicating if the point is 3D.
    """

    def __init__(self, x: Union['Point', Sequence, float], y: Optional[float] = None,
                 z: Optional[float] = None):
        """
        Initializes a Point object with x, y, and optionally z coordinates.

        Args:
            x (Union[int, float]): X-coordinate of the point.
            y (Union[int, float]): Y-coordinate of the point.
            z (Union[int, float], optional): Z-coordinate of the point. Defaults to 0.
        """
        if isinstance(x, Point):
            return  # If x is a Point instance, skip initialization

        try:
            if y is None:
                if len(x) in [2, 3]:
                    x, y, z = (*x, z) if len(x) == 2 else x
            elif not (isinstance(x, (int, float))
                      and isinstance(y, (int, float))
                      and (isinstance(z, (int, float)) or z is None)):
                raise ValueError
        except Exception:
            msg = x if y is None else f"({x}, {y}, {x})"
            raise ValueError(
                    f"A point must have 2 or 3 coordinates. Amount received {msg}")

        z, self._is_3d = (0.0, False) if z is None else (z, True)

        self.x = float(x)
        self.y = float(y)
        self.z = float(z)

    def __new__(cls, x: Union['Point', int, float], y: Union[int, float] = None, z: Union[int, float] = None):
        if isinstance(x, Point):
            return x  # Return the existing instance

        instance = super(Point, cls).__new__(cls)
        return instance  # Return a new instance

    @property
    def is_3d(self):
        return self._is_3d

    @property
    def magnitude(self) -> float:
        """Computes the magnitude (or norm) of the point when treated as a vector from the origin."""
        return math.sqrt(sum(coord ** 2 for coord in self))

    def dot(self, other: 'Point') -> float:
        """Computes the dot product between this point and another point."""
        return sum(a * b for a, b in zip(self, other))

    def cross(self, other: 'Point') -> 'Point':
        """Computes the cross product between this point and another point. Only applicable for 3D points."""
        return Point(
                self.y * other.z - self.z * other.y,
                self.z * other.x - self.x * other.z,
                self.x * other.y - self.y * other.x
        )

    @property
    def normalize(self) -> 'Point':
        """Returns a normalized version of the point (or vector)."""
        mag = self.magnitude
        return Point(self.x / mag, self.y / mag, self.z / mag)

    @property
    def coordinates(self) -> tuple:
        """Returns a tuple of the point's coordinates."""
        return self.x, self.y, self.z

    def distance_to(self, other: 'Point') -> float:
        """
        Computes the Euclidean distance between this point and another point.

        Args:
            other (Point): The other point to which the distance is calculated.

        Returns:
            float: The distance between the two points.
        """
        return (self - other).magnitude

    def angle_theta(self, other: 'Point', degrees=False) -> float:
        """
        Computes the angle (in radians or degrees) between this point and another point when treated as vectors.

        Args:
            other (Point): The other point (or vector) with which the angle is calculated.
            degrees (bool): if true returns the angle in degrees, default is False.

        Returns:
            float: The angle in radians or degress.
        """
        other = Point(other)
        dot_product = self.dot(other)
        magnitudes_product = self.magnitude * other.magnitude
        acos = math.acos(max(-1., min(1., dot_product / magnitudes_product)))
        return math.degrees(acos) if degrees else acos

    def rotation_angle(self, point, axis=0):
        """
        Creates vector v1 = self - point and calculates the angle of v1 with respect to an axis (x, y or z)
        @param point: Another point
        @param axis: x, y or z
        @return: Degrees
        """
        v1 = self - point

        if axis == 0:
            v2 = Point(1, 0, 0)  # Horizontal
        elif axis == 1:
            v2 = Point(0, 1, 0)  # Vertical
        else:
            v2 = Point(0, 0, 1)  # depth

        angle = v1.angle_theta(v2, True)
        return angle

    def bounding_box(self, w: int, h: int, position: str = "center") -> Tuple['Point', 'Point', 'Point', 'Point']:
        """
        Calculate the coordinates of a bounding box based on the given width, height, and position relative to a point.

        Args:
            w (int): Width of the bounding box.
            h (int): Height of the bounding box.
            position (str, optional): Position of the bounding box relative to the point.
                Valid positions: "center", "left-top", "left-bottom", "right_top", "right_bottom".
                Defaults to "center".

        Raises:
            ValueError: Raised when an invalid position is provided.

        Returns:
            Tuple[Point, Point, Point, Point]: A tuple containing four Point instances representing the
                coordinates of the bounding box in the order (start_x, start_y), (end_x, start_y),
                (end_x, end_y), and (start_x, end_y).
        """
        position = position.lower().strip()
        if position == "center":
            start_x, start_y = self.x - (w // 2), self.y - (h // 2)
        elif position == "left-top":
            start_x, start_y = self.x, self.y
        elif position == "left-bottom":
            start_x, start_y = self.x, self.y - (h // 2)
        elif position == "right_top":
            start_x, start_y = self.x - w, self.y
        elif position == "right_bottom":
            start_x, start_y = self.x - w, self.y - h
        else:
            raise ValueError('Invalid position. Enter a valid position for the point. Valid positions: ('
                             '"center", "left-top", "left-bottom", "right_top", "right_bottom")')

        end_x = start_x + w
        end_y = start_y + h

        return Point(start_x, start_y), Point(end_x, start_y), Point(end_x, end_y), Point(start_x, end_y)

    def rotate(self, angle: float, center=(0, 0, 0), axis: str = 'z') -> 'Point':
        """
        Rotates the point around the specified axis by the given angle.

        Args:
            angle (float): The rotation angle in degrees.
            center (Point): The rotation origin.
            axis (str, optional): The axis around which the point is rotated.
                                  Can be 'x', 'y', or 'z'. Default is 'z'.

        Returns:
            Point: A new point representing the rotated position.
        """
        angle = math.radians(angle)  # Convert angle to radians

        translated_point = self - center
        if axis == 'z':
            x_new = translated_point.x * math.cos(angle) - translated_point.y * math.sin(angle)
            y_new = translated_point.x * math.sin(angle) + translated_point.y * math.cos(angle)
            return Point(x_new, y_new, translated_point.z) + center

        elif axis == 'y':
            # Rotation around the Y axis
            x_new = translated_point.x * math.cos(angle) + translated_point.z * math.sin(angle)
            z_new = -translated_point.x * math.sin(angle) + translated_point.z * math.cos(angle)
            return Point(x_new, translated_point.y, z_new) + center

        elif axis == 'x':
            # Rotation around the X axis
            y_new = translated_point.y * math.cos(angle) - translated_point.z * math.sin(angle)
            z_new = translated_point.y * math.sin(angle) + translated_point.z * math.cos(angle)
            return Point(translated_point.x, y_new, z_new) + center

        else:
            raise ValueError(f"Invalid axis: {axis}. Axis should be 'x', 'y', or 'z'.")

    def __getitem__(self, item: Union[int, str]) -> Union[int, float]:
        """Allows indexing into the point to retrieve its coordinates."""
        if isinstance(item, str):
            return getattr(self, item.lower())
        return self.coordinates[item]

    def __iter__(self):
        """Returns an iterator over the point's coordinates."""
        return iter(self.coordinates)

    def __add__(self, other: 'Point') -> 'Point':
        """
        Implements the addition of two points.

        Args:
            other (Point): The other point to add.

        Returns:
            Point: A new point representing the sum of the two points' coordinates.
        """
        return Point(*(a + b for a, b in zip(self, other)))

    def __sub__(self, other: 'Point') -> 'Point':
        """
        Implements the subtraction of two points.

        Args:
            other (Point): The other point to subtract.

        Returns:
            Point: A new point representing the difference of the two points' coordinates.
        """
        return Point(*(a - b for a, b in zip(self, other)))

    def __repr__(self) -> str:
        """Returns the official string representation of the point."""
        return f"Point({self.x}, {self.y}, {self.z})"

    def __str__(self) -> str:
        """Returns the string representation of the point's coordinates."""
        return f"{self.coordinates}"

    def __mul__(self, scalar: Union[int, float]) -> 'Point':
        """Implements scalar multiplication of the point."""
        return Point(self.x * scalar, self.y * scalar, self.z * scalar)

    def __truediv__(self, scalar: Union[int, float]) -> 'Point':
        """Implements scalar division of the point."""
        return Point(self.x / scalar, self.y / scalar, self.z / scalar)

    def __eq__(self, other: Union['Point', Sequence]) -> bool:
        """
        Compares (==) two Point objects based on their coordinates.

        it's the same as a == b

        Args:
            other (Point): Another Point object.

        Returns:
            bool: True if their coordinates are identical, False otherwise.
        """
        other = Point(other)
        return self.coordinates == other.coordinates

    def __ne__(self, other: Union['Point', Sequence]) -> bool:
        """
        Checks inequality (!=) between two Point objects based on their coordinates.

        Args:
            other (Point): Another Point object.

        Returns:
            bool: True if their coordinates are different, False otherwise.
        """
        return not self == other

    def __lt__(self, other: Union['Point', Sequence]) -> bool:
        """
        Checks if the current Point object is "less than" (<) another Point object.

        The comparison is primarily based on the magnitude. If magnitudes are equal,
        lexicographical comparison is used (i.e., x first, then y, then z).

        Args:
            other (Point): Another Point object.

        Returns:
            bool: True if the current point is "less than" the other point, False otherwise.
            ValueError if the other object is not a Point.
        """
        other = Point(other)
        return (self.magnitude, *self.coordinates) < (other.magnitude, *other.coordinates)

    def __le__(self, other: Union['Point', Sequence]) -> bool:
        """
        Checks if the current Point object is "less than or equal to" (<=) another Point object.

        The comparison is primarily based on the magnitude. If magnitudes are equal,
        lexicographical comparison is used (i.e., x first, then y, then z).

        Args:
            other (Point): Another Point object.

        Returns:
            bool: True if the current point is "less than or equal to" the other point, False otherwise.
            ValueError if the other object is not a Point.
        """
        other = Point(other)
        return (self.magnitude, *self.coordinates) <= (other.magnitude, *other.coordinates)

    def __gt__(self, other: Union['Point', Sequence]) -> bool:
        """
        Checks if the current Point object is "greater than" (>) another Point object.

        The comparison is primarily based on the magnitude. If magnitudes are equal,
        lexicographical comparison is used (i.e., x first, then y, then z).

        Args:
            other (Point): Another Point object.

        Returns:
            bool: True if the current point is "greater than" the other point, False otherwise.
            ValueError if the other object is not a Point.
        """
        other = Point(other)
        return (self.magnitude, *self.coordinates) > (other.magnitude, *other.coordinates)

    def __ge__(self, other: Union['Point', Sequence]) -> bool:
        """
        Checks if the current Point object is "greater than or equal to" another Point object.

        The comparison is primarily based on the magnitude. If magnitudes are equal,
        lexicographical comparison is used (i.e., x first, then y, then z).

        Args:
            other (Point): Another Point object.

        Returns:
            bool: True if the current point is "greater than or equal to" (>=) the other point, False otherwise.
            ValueError if the other object is not a Point.
        """
        other = Point(other)
        return (self.magnitude, *self.coordinates) >= (other.magnitude, *other.coordinates)



"""

Copyright (c) 2019 The Cereja Project

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from ..system.unicode import Unicode
import string

NUMBER_WORDS = {
    'pt':
        {
            'um':         1, 'dois': 2, 'três': 3, 'quatro': 4, 'cinco': 5, 'seis': 6, 'sete': 7, 'oito': 8,
            'nove':       9,
            'dez':        10, 'onze': 11, 'doze': 12, 'treze': 13, 'catorze': 14, 'quinze': 15, 'dezesseis': 16,
            'dezessete':  17, 'dezoito': 18, 'dezenove': 19,
            'vinte':      20, 'trinta': 30, 'quarenta': 40, 'cinquenta': 50, 'sessenta': 60, 'setenta': 70,
            'oitenta':    80, 'noventa': 90,
            'cem':        100, 'cento': 100, 'duzentos': 200, 'trezentos': 300, 'quatrocentos': 400,
            'quinhentos': 500,
            'seiscentos': 600, 'setecentos': 700, 'oitocentos': 800, 'novecentos': 900,
            'mil':        1000, 'milhão': 1000000, 'bilhão': 1000000000, 'trilhão': 1000000000000
        },
    'en':
        {
            'one':       1, 'two': 2, 'three': 3, 'four': 4, 'five': 5, 'six': 6, 'seven': 7, 'eight': 8, 'nine': 9,
            'ten':       10, 'eleven': 11, 'twelve': 12, 'thirteen': 13, 'fourteen': 14, 'fifteen': 15, 'sixteen': 16,
            'seventeen': 17, 'eighteen': 18, 'nineteen': 19,
            'twenty':    20, 'thirty': 30, 'forty': 40, 'fifty': 50, 'sixty': 60, 'seventy': 70, 'eighty': 80,
            'ninety':    90,
            'hundred':   100, 'thousand': 1000, 'million': 1000000, 'billion': 1000000000, 'trillion': 1000000000000
        }}

ENG_CONTRACTIONS = {
    "ain't":        "am not,are not,is not,has not,have not",
    "aren't":       "are not,am not",
    "can't":        "cannot",
    "can't've":     "cannot have",
    "'cause":       "because",
    "could've":     "could have",
    "couldn't":     "could not",
    "couldn't've":  "could not have",
    "didn't":       "did not",
    "doesn't":      "does not",
    "don't":        "do not",
    "hadn't":       "had not",
    "hadn't've":    "had not have",
    "hasn't":       "has not",
    "haven't":      "have not",
    "he'd":         "he had,he would",
    "he'd've":      "he would have",
    "he'll":        "he shall,he will",
    "he'll've":     "he shall have,he will have",
    "he's":         "he has,he is",
    "how'd":        "how did",
    "how'd'y":      "how do you",
    "how'll":       "how will",
    "how's":        "how has,how is,how does",
    "i'd":          "i had,i would",
    "i'd've":       "i would have",
    "i'll":         "i shall,i will",
    "i'll've":      "i shall have,i will have",
    "i'm":          "i am",
    "i've":         "i have",
    "isn't":        "is not",
    "it'd":         "it had,it would",
    "it'd've":      "it would have",
    "it'll":        "it shall,it will",
    "it'll've":     "it shall have,it will have",
    "it's":         "it has,it is",
    "let's":        "let us",
    "ma'am":        "madam",
    "mayn't":       "may not",
    "might've":     "might have",
    "mightn't":     "might not",
    "mightn't've":  "might not have",
    "must've":      "must have",
    "mustn't":      "must not",
    "mustn't've":   "must not have",
    "needn't":      "need not",
    "needn't've":   "need not have",
    "o'clock":      "of the clock",
    "oughtn't":     "ought not",
    "oughtn't've":  "ought not have",
    "shan't":       "shall not",
    "sha'n't":      "shall not",
    "shan't've":    "shall not have",
    "she'd":        "she had,she would",
    "she'd've":     "she would have",
    "she'll":       "she shall,she will",
    "she'll've":    "she shall have,she will have",
    "she's":        "she has,she is",
    "should've":    "should have",
    "shouldn't":    "should not",
    "shouldn't've": "should not have",
    "so've":        "so have",
    "so's":         "so as,so is",
    "that'd":       "that would,that had",
    "that'd've":    "that would have",
    "that's":       "that has,that is",
    "there'd":      "there had,there would",
    "there'd've":   "there would have",
    "there's":      "there has,there is",
    "they'd":       "they had,they would",
    "they'd've":    "they would have",
    "they'll":      "they shall,they will",
    "they'll've":   "they shall have,they will have",
    "they're":      "they are",
    "they've":      "they have",
    "to've":        "to have",
    "wasn't":       "was not",
    "we'd":         "we had,we would",
    "we'd've":      "we would have",
    "we'll":        "we will",
    "we'll've":     "we will have",
    "we're":        "we are",
    "we've":        "we have",
    "weren't":      "were not",
    "what'll":      "what shall,what will",
    "what'll've":   "what shall have,what will have",
    "what're":      "what are",
    "what's":       "what has,what is",
    "what've":      "what have",
    "when's":       "when has,when is",
    "when've":      "when have",
    "where'd":      "where did",
    "where's":      "where has,where is",
    "where've":     "where have",
    "who'll":       "who shall,who will",
    "who'll've":    "who shall have,who will have",
    "who's":        "who has,who is",
    "who've":       "who have",
    "why's":        "why has,why is",
    "why've":       "why have",
    "will've":      "will have",
    "won't":        "will not",
    "won't've":     "will not have",
    "would've":     "would have",
    "wouldn't":     "would not",
    "wouldn't've":  "would not have",
    "y'all":        "you all",
    "y'all'd":      "you all would",
    "y'all'd've":   "you all would have",
    "y'all're":     "you all are",
    "y'all've":     "you all have",
    "you'd":        "you had,you would",
    "you'd've":     "you would have",
    "you'll":       "you shall,you will",
    "you'll've":    "you shall have,you will have",
    "you're":       "you are",
    "you've":       "you have",
}
PUNCTUATION = [
    ",",
    "!",
    "#",
    "$",
    "%",
    "'",
    "*",
    "+",
    "-",
    ".",
    "/",
    "?",
    "@",
    "\\",
    "^",
    "_",
    "~",
]
STOP_WORDS = {
    "english":    ["is", "are", "am", "at", "a", "an", "of", "the"],
    "portuguese": [
        "em",
        "do",
        "da",
        "de",
        "pro",
        "pra",
        "no",
        "dos",
        "na",
        "os",
        "à",
        "pela",
        "nas",
        "num",
        "das",
        "numa",
        "nos",
        "às",
        "pelas",
        "as",
        "uns",
        "umas",
        "lhe",
        "é",
        "o",
        "a",
    ],
}
LANGUAGES = {"english", "portuguese"}
VALID_LANGUAGE_CHAR = {chr(i) for i in range(10000) if "LETTER" in Unicode(i).name}
VALID_LANGUAGE_CHAR.update(string.digits)
VALID_LANGUAGE_CHAR.update(",!%$?-.")
VALID_LANGUAGE_CHAR.update(" ")

DATA_UNIT_MAP = {"B": 1.0e0, "KB": 1.0e3, "MB": 1.0e6, "GB": 1.0e9, "TB": 1.0e12}

PROXIES_URL = "https://raw.githubusercontent.com/jetkai/proxy-list/main/online-proxies/json/proxies.json"



import unittest
from cereja import hashtools


class HashtoolsTest(unittest.TestCase):
    def test_base64_decode(self):
        self.assertEqual(
            hashtools.base64_decode(b"YWJjZGVmIGcgaCBpIGo=", eval_str=True),
            "abcdef g h i j",
        )
        self.assertEqual(
            hashtools.base64_decode(b"eydvaSc6ICd0dWRvIGJlbSd9", eval_str=True),
            {"oi": "tudo bem"},
        )
        self.assertEqual(
            hashtools.base64_decode(b"YWJjZGVmIGcgaCBpIGo="), b"abcdef g h i j"
        )

    def test_base64_encode(self):
        self.assertEqual(
            hashtools.base64_encode("abcdef g h i j"), b"YWJjZGVmIGcgaCBpIGo="
        )
        self.assertEqual(
            hashtools.base64_encode({"oi": "tudo bem"}), b"eydvaSc6ICd0dWRvIGJlbSd9"
        )

    def test_is_base64(self):
        pass

    def test_md5(self):
        pass


if __name__ == "__main__":
    unittest.main()
